@@ -20,10 +20,11 @@
 //!
 //! [IETF-RFC6690 CoAP link-formats]: https://tools.ietf.org/html/rfc6690
 
-use super::*;
 use alloc::{borrow::Cow, string::ToString};
-use core::fmt::{Display, Write};
-use core::iter::FusedIterator;
+use core::{
+    fmt::{Display, Write},
+    iter::FusedIterator,
+};
 
 /// Relation Type.
 ///
@@ -265,7 +266,7 @@ impl<'a> Iterator for LinkFormatParser<'a> {
         let link_ref = iter.as_str();
 
         // Proceed through characters until we get a '>'.
-        while let Some(c) = iter.next() {
+        for c in iter.by_ref() {
             if c == '>' {
                 break;
             }
