@@ -5,7 +5,7 @@ use alloc::{
 };
 use core::{fmt::Display, marker::PhantomData};
 
-use super::{request::CoapRequest, MessageClass, MessageType, Packet};
+use crate::{request::CoapRequest, MessageClass, MessageType, Packet};
 
 const DEFAULT_UNACKNOWLEDGED_LIMIT: u8 = 10;
 
@@ -44,7 +44,7 @@ impl<Endpoint: Display + PartialEq + Clone> Subject<Endpoint> {
 
         let observer = Observer {
             endpoint: observer_endpoint.clone(),
-            token: token.clone(),
+            token: token.to_vec(),
             unacknowledged_messages: 0,
             message_id: None,
         };
@@ -72,7 +72,7 @@ impl<Endpoint: Display + PartialEq + Clone> Subject<Endpoint> {
         }
     }
 
-    // Removes an observer from the interested resource.
+    /// Removes an observer from the interested resource.
     pub fn deregister(&mut self, request: &CoapRequest<Endpoint>) {
         let observer_endpoint = request.source.as_ref().unwrap();
         let resource_path = request.get_path();
@@ -127,8 +127,8 @@ impl<Endpoint: Display + PartialEq + Clone> Subject<Endpoint> {
         for (resource_path, resource) in self.resources.iter_mut() {
             let observer = resource.observers.iter_mut().find(|x| {
                 if let Some(observe_msg_id) = x.message_id {
-                    // Unacknowledgement doesn't officially require the token to be passed in the ACK
-                    // so it's not checked
+                    // Unacknowledgement doesn't officially require the token
+                    // to be passed in the ACK so it's not checked
                     return x.endpoint == *observer_endpoint
                         && observe_msg_id == message_id;
                 }
@@ -181,14 +181,7 @@ pub fn create_notification(
     packet.header.message_id = message_id;
     packet.set_token(token);
     packet.payload = payload;
-
-    let mut sequence_bytes = sequence.to_be_bytes().to_vec();
-    let first_non_zero = sequence_bytes
-        .iter()
-        .position(|&x| x > 0)
-        .unwrap_or(sequence_bytes.len());
-    sequence_bytes.drain(0..first_non_zero);
-    packet.set_observe(sequence_bytes);
+    packet.set_observe_value(sequence);
 
     packet
 }
@@ -205,12 +198,10 @@ impl<Endpoint: Display + PartialEq + Clone> Default for Subject<Endpoint> {
 
 #[cfg(test)]
 mod test {
-    use super::{
-        super::{
-            header::{MessageType, RequestType as Method},
-            packet::ObserveOption,
-        },
-        *,
+    use super::*;
+    use crate::{
+        header::{MessageType, RequestType as Method},
+        packet::ObserveOption,
     };
 
     type Endpoint = String;
@@ -224,9 +215,7 @@ mod test {
         request.set_method(Method::Get);
         request.set_path(resource_path.clone());
         request.message.set_token(vec![0x7d, 0x34]);
-        request
-            .message
-            .set_observe(vec![ObserveOption::Register as u8]);
+        request.set_observe_flag(ObserveOption::Register);
 
         let mut subject: Subject<Endpoint> = Subject::default();
         subject.register(&request);
@@ -247,18 +236,14 @@ mod test {
         request1.set_method(Method::Get);
         request1.set_path(resource_path.clone());
         request1.message.set_token(vec![0x00, 0x00]);
-        request1
-            .message
-            .set_observe(vec![ObserveOption::Register as u8]);
+        request1.set_observe_flag(ObserveOption::Register);
 
         let mut request2 = CoapRequest::new();
         request2.source = Some(String::from("0.0.0.0"));
         request2.set_method(Method::Get);
         request2.set_path(resource_path.clone());
         request2.message.set_token(vec![0xff, 0xff]);
-        request2
-            .message
-            .set_observe(vec![ObserveOption::Register as u8]);
+        request2.set_observe_flag(ObserveOption::Register);
 
         let mut subject: Subject<Endpoint> = Subject::default();
         subject.register(&request1);
@@ -284,9 +269,7 @@ mod test {
         request1.set_method(Method::Get);
         request1.set_path(resource_path.clone());
         request1.message.set_token(vec![0x00, 0x00]);
-        request1
-            .message
-            .set_observe(vec![ObserveOption::Register as u8]);
+        request1.set_observe_flag(ObserveOption::Register);
 
         let mut subject: Subject<Endpoint> = Subject::default();
         subject.register(&request1);
@@ -334,9 +317,7 @@ mod test {
         request1.set_method(Method::Get);
         request1.set_path(resource_path.clone());
         request1.message.set_token(vec![0x00, 0x00]);
-        request1
-            .message
-            .set_observe(vec![ObserveOption::Register as u8]);
+        request1.set_observe_flag(ObserveOption::Register);
 
         let mut subject: Subject<Endpoint> = Subject::default();
         subject.set_unacknowledged_limit(5);
