@@ -4,9 +4,13 @@ use alloc::{
 };
 use core::convert::TryFrom;
 
-use super::{
-    error::{InvalidContentFormat, InvalidObserve, MessageError},
+use crate::{
+    error::{
+        IncompatibleOptionValueFormat, InvalidContentFormat, InvalidObserve,
+        MessageError,
+    },
     header::{Header, HeaderRaw, MessageClass},
+    option_value::{OptionValueType, OptionValueU16, OptionValueU32},
 };
 
 macro_rules! u8_to_unsigned_be {
@@ -228,7 +232,7 @@ impl Packet {
     }
 
     /// Returns the token.
-    pub fn get_token(&self) -> &Vec<u8> {
+    pub fn get_token(&self) -> &[u8] {
         &self.token
     }
 
@@ -237,11 +241,53 @@ impl Packet {
         self.options.insert(tp.into(), value);
     }
 
+    /// Sets an option's values using a structured option value format.
+    pub fn set_options_as<T: OptionValueType>(
+        &mut self,
+        tp: CoapOption,
+        value: LinkedList<T>,
+    ) {
+        let raw_value = value.into_iter().map(|x| x.into()).collect();
+        self.set_option(tp, raw_value);
+    }
+
     /// Returns an option's values.
     pub fn get_option(&self, tp: CoapOption) -> Option<&LinkedList<Vec<u8>>> {
         self.options.get(&tp.into())
     }
 
+    /// Returns an option's values all decoded using the specified structured
+    /// option value format.
+    pub fn get_options_as<T: OptionValueType>(
+        &self,
+        tp: CoapOption,
+    ) -> Option<LinkedList<Result<T, IncompatibleOptionValueFormat>>> {
+        self.get_option(tp).map(|options| {
+            options
+                .iter()
+                .map(|raw_value| T::try_from(raw_value.clone()))
+                .collect()
+        })
+    }
+
+    /// Returns an option's first value as a convenience when only one is
+    /// expected.
+    pub fn get_first_option(&self, tp: CoapOption) -> Option<&Vec<u8>> {
+        self.options
+            .get(&tp.into())
+            .and_then(|options| options.front())
+    }
+
+    /// Returns an option's first value as a convenience when only one is
+    /// expected.
+    pub fn get_first_option_as<T: OptionValueType>(
+        &self,
+        tp: CoapOption,
+    ) -> Option<Result<T, IncompatibleOptionValueFormat>> {
+        self.get_first_option(tp)
+            .map(|value| T::try_from(value.clone()))
+    }
+
     /// Adds an option value.
     pub fn add_option(&mut self, tp: CoapOption, value: Vec<u8>) {
         let num = tp.into();
@@ -255,6 +301,15 @@ impl Packet {
         self.options.insert(num, list);
     }
 
+    /// Adds an option value using a structured option value format.
+    pub fn add_option_as<T: OptionValueType>(
+        &mut self,
+        tp: CoapOption,
+        value: T,
+    ) {
+        self.add_option(tp, value.into());
+    }
+
     /// Removes an option.
     pub fn clear_option(&mut self, tp: CoapOption) {
         if let Some(list) = self.options.get_mut(&tp.into()) {
@@ -264,49 +319,33 @@ impl Packet {
 
     /// Sets the content-format.
     pub fn set_content_format(&mut self, cf: ContentFormat) {
-        let content_format: usize = cf.into();
-        let msb = (content_format >> 8) as u8;
-        let lsb = (content_format & 0xFF) as u8;
-
-        let content_format: Vec<u8> =
-            if msb == 0 { vec![lsb] } else { vec![msb, lsb] };
-
-        self.add_option(CoapOption::ContentFormat, content_format);
+        let content_format: u16 = u16::try_from(usize::from(cf)).unwrap();
+        self.add_option_as(
+            CoapOption::ContentFormat,
+            OptionValueU16(content_format),
+        );
     }
 
     /// Returns the content-format.
     pub fn get_content_format(&self) -> Option<ContentFormat> {
-        if let Some(list) = self.get_option(CoapOption::ContentFormat) {
-            if let Some(vector) = list.front() {
-                if vector.len() == 0 {
-                    return None;
-                }
-
-                let number =
-                    vector.iter().fold(0, |acc, &b| (acc << 8) + b as u16);
-
-                return ContentFormat::try_from(number as usize).ok();
-            }
-        }
-
-        None
+        self.get_first_option_as::<OptionValueU16>(CoapOption::ContentFormat)
+            .and_then(|option| option.ok())
+            .map(|value| usize::try_from(value.0).unwrap())
+            .and_then(|value| ContentFormat::try_from(value).ok())
     }
 
     /// Sets the value of the observe option.
-    pub fn set_observe(&mut self, value: Vec<u8>) {
+    pub fn set_observe_value(&mut self, value: u32) {
         self.clear_option(CoapOption::Observe);
-        self.add_option(CoapOption::Observe, value);
+        self.add_option_as(CoapOption::Observe, OptionValueU32(value));
     }
 
     /// Returns the value of the observe option.
-    pub fn get_observe(&self) -> Option<&Vec<u8>> {
-        if let Some(list) = self.get_option(CoapOption::Observe) {
-            if let Some(flag) = list.front() {
-                return Some(flag);
-            }
-        }
-
-        None
+    pub fn get_observe_value(
+        &self,
+    ) -> Option<Result<u32, IncompatibleOptionValueFormat>> {
+        self.get_first_option_as::<OptionValueU32>(CoapOption::Observe)
+            .map(|option| option.map(|value| value.0))
     }
 
     /// Decodes a byte slice and constructs the equivalent packet.
@@ -559,7 +598,9 @@ impl Packet {
 
 #[cfg(test)]
 mod test {
-    use super::{super::header, *};
+    use super::*;
+    use crate::{header, option_value::OptionValueString};
+    use alloc::borrow::ToOwned;
 
     #[test]
     fn test_decode_packet_with_options() {
@@ -747,11 +788,40 @@ mod test {
         assert_eq!(3, pp.options().len());
     }
 
+    #[test]
+    fn test_option_u32_format() {
+        let mut p = Packet::new();
+        let option_key = CoapOption::Observe;
+        let values = vec![0, 100, 1000, 10000, u32::MAX];
+        for &value in &values {
+            p.add_option_as(option_key, OptionValueU32(value));
+        }
+        let expected = values.iter().map(|&x| Ok(OptionValueU32(x))).collect();
+        let actual = p.get_options_as::<OptionValueU32>(option_key);
+        assert_eq!(actual, Some(expected));
+    }
+
+    #[test]
+    fn test_option_utf8_format() {
+        let mut p = Packet::new();
+        let option_key = CoapOption::UriPath;
+        let values = vec!["", "simple", "unicode 😁 stuff"];
+        for &value in &values {
+            p.add_option_as(option_key, OptionValueString(value.to_owned()));
+        }
+        let expected = values
+            .iter()
+            .map(|&x| Ok(OptionValueString(x.to_owned())))
+            .collect();
+        let actual = p.get_options_as::<OptionValueString>(option_key);
+        assert_eq!(actual, Some(expected));
+    }
+
     #[test]
     fn observe() {
         let mut p = Packet::new();
-        assert_eq!(None, p.get_observe());
-        p.set_observe(vec![0]);
-        assert_eq!(Some(&vec![0]), p.get_observe());
+        assert_eq!(None, p.get_observe_value());
+        p.set_observe_value(0);
+        assert_eq!(Some(Ok(0)), p.get_observe_value());
     }
 }
