@@ -2,8 +2,10 @@ use alloc::{
     string::{String, ToString},
     vec::Vec,
 };
+use core::convert::TryFrom;
 
-use super::{
+use crate::{
+    error::InvalidObserve,
     header::{MessageClass, RequestType as Method},
     packet::{CoapOption, ObserveOption, Packet},
     response::CoapResponse,
@@ -85,21 +87,24 @@ impl<Endpoint> CoapRequest<Endpoint> {
         }
     }
 
-    /// Returns the flag in the Observe option.
-    pub fn get_observe_flag(&self) -> Option<ObserveOption> {
-        self.message
-            .get_observe()
-            .and_then(|option| match option.get(0) {
-                Some(&x) if x == ObserveOption::Register as u8 => {
-                    Some(ObserveOption::Register)
-                }
-                Some(&x) if x == ObserveOption::Deregister as u8 => {
-                    Some(ObserveOption::Deregister)
-                }
-                Some(_) => None,
-                // Value is Register by default if not present
-                None => Some(ObserveOption::Register),
-            })
+    /// Returns the flag in the Observe option or InvalidObserve if the flag
+    /// was provided but not understood.
+    pub fn get_observe_flag(
+        &self,
+    ) -> Option<Result<ObserveOption, InvalidObserve>> {
+        self.message.get_observe_value().map(|observe| {
+            observe
+                .map(|value| usize::try_from(value).unwrap())
+                .map_or(Err(InvalidObserve), |value| {
+                    ObserveOption::try_from(value)
+                })
+        })
+    }
+
+    /// Sets the flag in the Observe option.
+    pub fn set_observe_flag(&mut self, flag: ObserveOption) {
+        let value = u32::try_from(usize::from(flag)).unwrap();
+        self.message.set_observe_value(value);
     }
 }
 
@@ -115,7 +120,8 @@ impl<Endpoint> Default for CoapRequest<Endpoint> {
 
 #[cfg(test)]
 mod test {
-    use super::{super::header::MessageType, *};
+    use super::*;
+    use crate::header::MessageType;
 
     struct Endpoint(String);
 
@@ -225,4 +231,26 @@ mod test {
         request.set_path(path3);
         assert_eq!(path3, request.get_path());
     }
+
+    #[test]
+    fn test_unknown_observe_flag() {
+        let mut request: CoapRequest<Endpoint> = CoapRequest::new();
+
+        request.message.set_observe_value(32);
+        let expected = Some(Err(InvalidObserve));
+        let actual = request.get_observe_flag();
+        assert_eq!(actual, expected);
+    }
+
+    #[test]
+    fn test_garbage_in_observe_field() {
+        let mut request: CoapRequest<Endpoint> = CoapRequest::new();
+
+        request
+            .message
+            .add_option(CoapOption::Observe, b"bunch of nonsense".to_vec());
+        let expected = Some(Err(InvalidObserve));
+        let actual = request.get_observe_flag();
+        assert_eq!(actual, expected);
+    }
 }
